@@ -208,7 +208,7 @@ public:
 
     unsigned wasted();
 
-    int cmp(const ptr& o);
+    int cmp(const ptr& o) const;
     bool is_zero() const;
 
     // modifiers
@@ -448,6 +448,7 @@ public:
 
   public:
     hash() : crc(0) { }
+    hash(uint32_t init) : crc(init) { }
 
     void update(buffer::list& bl) {
       crc = bl.crc32c(crc);
