@@ -236,7 +236,7 @@ int librados::RadosClient::connect()
   ldout(cct, 1) << "starting objecter" << dendl;
 
   err = -ENOMEM;
-  objecter = new Objecter(cct, messenger, &monclient, &osdmap, lock, timer,
+  objecter = new (std::nothrow) Objecter(cct, messenger, &monclient, &osdmap, lock, timer,
 			  cct->_conf->rados_mon_op_timeout,
 			  cct->_conf->rados_osd_op_timeout);
   if (!objecter)
@@ -287,8 +287,19 @@ int librados::RadosClient::connect()
   err = 0;
 
  out:
-  if (err)
+  if (err) {
     state = DISCONNECTED;
+
+    if (objecter) {
+      delete objecter;
+      objecter = NULL;
+    }
+    if (messenger) {
+      delete messenger;
+      messenger = NULL;
+    }
+  }
+
   return err;
 }
 
@@ -434,6 +445,10 @@ int librados::RadosClient::wait_for_osdmap()
 {
   assert(lock.is_locked());
 
+  if (state != CONNECTED) {
+    return -ENOTCONN;
+  }
+
   utime_t timeout;
   if (cct->_conf->rados_mon_op_timeout > 0)
     timeout.set_from_double(cct->_conf->rados_mon_op_timeout);
