@@ -28,12 +28,13 @@ class WritebackHandler {
    * @param snapid read snapid
    */
   virtual bool may_copy_on_write(const object_t& oid, uint64_t read_off, uint64_t read_len, snapid_t snapid) = 0;
-  virtual tid_t write(const object_t& oid, const object_locator_t& oloc,
-		      uint64_t off, uint64_t len, const SnapContext& snapc,
-		      const bufferlist &bl, utime_t mtime, uint64_t trunc_size,
-		      __u32 trunc_seq, Context *oncommit) = 0;
-  virtual tid_t lock(const object_t& oid, const object_locator_t& oloc, int op,
-		     int flags, Context *onack, Context *oncommit) {
+  virtual ceph_tid_t write(const object_t& oid, const object_locator_t& oloc,
+			   uint64_t off, uint64_t len, const SnapContext& snapc,
+			   const bufferlist &bl, utime_t mtime,
+			   uint64_t trunc_size, __u32 trunc_seq,
+			   Context *oncommit) = 0;
+  virtual ceph_tid_t lock(const object_t& oid, const object_locator_t& oloc,
+			  int op, int flags, Context *onack, Context *oncommit) {
     assert(0 == "this WritebackHandler does not support the lock operation");
   }
 };
